@@ -32,6 +32,7 @@
 #include "StaticContentHandler.h"
 #include "Configuration.h"
 #include "cxx_supportlib/Constants.h"
+#include "cxx_supportlib/FileTools/PathManipCBindings.h"
 
 
 #define NGX_HTTP_SCGI_PARSE_NO_HEADER  20
@@ -175,12 +176,12 @@ find_base_uri(ngx_http_request_t *r, const passenger_loc_conf_t *loc,
     ngx_uint_t  i;
     ngx_str_t  *base_uris, *base_uri, *uri;
 
-    if (loc->base_uris == NGX_CONF_UNSET_PTR) {
+    if (loc->autogenerated.base_uris == NGX_CONF_UNSET_PTR) {
         return 0;
     } else {
-        base_uris = (ngx_str_t *) loc->base_uris->elts;
+        base_uris = (ngx_str_t *) loc->autogenerated.base_uris->elts;
         uri       = &r->uri;
-        for (i = 0; i < loc->base_uris->nelts; i++) {
+        for (i = 0; i < loc->autogenerated.base_uris->nelts; i++) {
             base_uri = &base_uris[i];
 
             if (base_uri->len == 1 && base_uri->data[0] == '/') {
@@ -501,11 +502,11 @@ construct_request_buffer(ngx_http_request_t *r, passenger_loc_conf_t *slcf,
         } while (0)
 
     ngx_uint_t       total_size = 0;
-    ngx_str_t       *union_station_filters;
     ngx_uint_t       i;
     ngx_list_part_t *part;
     ngx_table_elt_t *header;
     size_t           len;
+    ngx_str_t        public_dir_parent;
     ngx_http_script_len_code_pt lcode;
     ngx_http_script_code_pt     code;
     ngx_http_script_engine_t    e, le;
@@ -678,21 +679,34 @@ construct_request_buffer(ngx_http_request_t *r, passenger_loc_conf_t *slcf,
         PUSH_STATIC_STR("\r\n");
     }
 
-    if (slcf->app_group_name.data == NULL) {
+    if (slcf->autogenerated.app_group_name.data == NULL) {
         PUSH_STATIC_STR("!~PASSENGER_APP_GROUP_NAME: ");
-        if (b != NULL) {
-            b->last = ngx_copy(b->last, context->public_dir.data,
-                context->public_dir.len);
+        if (slcf->autogenerated.app_root.data == NULL) {
+            public_dir_parent.data = (u_char *) psg_extract_dir_name_static(
+                (const char *) context->public_dir.data,
+                context->public_dir.len,
+                &public_dir_parent.len);
+            if (b != NULL) {
+                b->last = ngx_copy(b->last, public_dir_parent.data,
+                    public_dir_parent.len);
+            }
+            total_size += public_dir_parent.len;
+        } else {
+            if (b != NULL) {
+                b->last = ngx_copy(b->last,
+                    slcf->autogenerated.app_root.data,
+                    slcf->autogenerated.app_root.len);
+            }
+            total_size += slcf->autogenerated.app_root.len;
         }
-        total_size += context->public_dir.len;
-        if (slcf->environment.data != NULL) {
+        if (slcf->autogenerated.environment.data != NULL) {
             if (b != NULL) {
                 b->last = ngx_copy(b->last, " (", 2);
-                b->last = ngx_copy(b->last, slcf->environment.data,
-                    slcf->environment.len);
+                b->last = ngx_copy(b->last, slcf->autogenerated.environment.data,
+                    slcf->autogenerated.environment.len);
                 b->last = ngx_copy(b->last, ")", 1);
             }
-            total_size += (sizeof(" (") - 1) + slcf->environment.len + (sizeof(")") - 1);
+            total_size += (sizeof(" (") - 1) + slcf->autogenerated.environment.len + (sizeof(")") - 1);
         }
         PUSH_STATIC_STR("\r\n");
     }
@@ -705,21 +719,6 @@ construct_request_buffer(ngx_http_request_t *r, passenger_loc_conf_t *slcf,
     total_size += state->app_type.len;
     PUSH_STATIC_STR("\r\n");
 
-    if (slcf->union_station_filters != NGX_CONF_UNSET_PTR
-     && slcf->union_station_filters->nelts > 0)
-    {
-        union_station_filters = (ngx_str_t *) slcf->union_station_filters->elts;
-        for (i = 0; i < slcf->union_station_filters->nelts; i++) {
-            PUSH_STATIC_STR("!~UNION_STATION_FILTERS: ");
-            if (b != NULL) {
-                b->last = ngx_copy(b->last, union_station_filters[i].data,
-                    union_station_filters[i].len);
-            }
-            total_size += union_station_filters[i].len;
-            PUSH_STATIC_STR("\r\n");
-        }
-    }
-
     if (b != NULL) {
         b->last = ngx_copy(b->last, slcf->options_cache.data, slcf->options_cache.len);
     }
@@ -1199,7 +1198,7 @@ process_header(ngx_http_request_t *r)
 
                 h->key.len = sizeof("Server") - 1;
                 h->key.data = (u_char *) "Server";
-                if (!passenger_main_conf.show_version_in_header) {
+                if (!passenger_main_conf.autogenerated.show_version_in_header) {
                     if (clcf->server_tokens) {
                         h->value.data = (u_char *) (NGINX_VER " + " PROGRAM_NAME);
                     } else {
@@ -1327,7 +1326,7 @@ passenger_content_handler(ngx_http_request_t *r)
     passenger_context_t   *context;
     PP_Error               error;
 
-    if (passenger_main_conf.root_dir.len == 0) {
+    if (passenger_main_conf.autogenerated.root_dir.len == 0) {
         return NGX_DECLINED;
     }
 
@@ -1336,7 +1335,7 @@ passenger_content_handler(ngx_http_request_t *r)
     /* Let the next content handler take care of this request if Phusion
      * Passenger is disabled for this URL.
      */
-    if (!slcf->enabled) {
+    if (!slcf->autogenerated.enabled) {
         return NGX_DECLINED;
     }
 
@@ -1369,11 +1368,11 @@ passenger_content_handler(ngx_http_request_t *r)
         /* Store the found base URI into context->public_dir. We infer that
          * the 'public' directory of the web app equals document root + base URI.
          */
-        if (slcf->document_root.data != NULL) {
-            len = slcf->document_root.len + 1;
+        if (slcf->autogenerated.document_root.data != NULL) {
+            len = slcf->autogenerated.document_root.len + 1;
             context->public_dir.data = ngx_palloc(r->pool, sizeof(u_char) * len);
-            end = ngx_copy(context->public_dir.data, slcf->document_root.data,
-                           slcf->document_root.len);
+            end = ngx_copy(context->public_dir.data, slcf->autogenerated.document_root.data,
+                           slcf->autogenerated.document_root.len);
         } else {
             len = root_path.len + base_uri.len + 1;
             context->public_dir.data = ngx_palloc(r->pool, sizeof(u_char) * len);
@@ -1395,6 +1394,13 @@ passenger_content_handler(ngx_http_request_t *r)
         *end = '\0';
         context->public_dir.len  = root_path.len;
     }
+    if (context->public_dir.len == 0) {
+        /* If the `root` directive is set to `/` then `public_dir`
+         * becomes the empty string. We fix this into `/`.
+         */
+        context->public_dir.data = (u_char *) "/";
+        context->public_dir.len = 1;
+    }
 
     /* If there's a corresponding page cache file for this URL, then serve that
      * file instead.
@@ -1406,9 +1412,9 @@ passenger_content_handler(ngx_http_request_t *r)
         return passenger_static_content_handler(r, &page_cache_file);
     }
 
-    if (slcf->app_type.data == NULL) {
+    if (slcf->autogenerated.app_type.data == NULL) {
         pp_error_init(&error);
-        if (slcf->app_root.data == NULL) {
+        if (slcf->autogenerated.app_root.data == NULL) {
             context->app_type = pp_app_type_detector_check_document_root(
                 pp_app_type_detector,
                 (const char *) context->public_dir.data, context->public_dir.len,
@@ -1417,7 +1423,7 @@ passenger_content_handler(ngx_http_request_t *r)
         } else {
             context->app_type = pp_app_type_detector_check_app_root(
                 pp_app_type_detector,
-                (const char *) slcf->app_root.data, slcf->app_root.len,
+                (const char *) slcf->autogenerated.app_root.data, slcf->autogenerated.app_root.len,
                 &error);
         }
         if (context->app_type == PAT_NONE) {
@@ -1442,8 +1448,8 @@ passenger_content_handler(ngx_http_request_t *r)
             return NGX_HTTP_INTERNAL_SERVER_ERROR;
         }
     } else {
-        context->app_type = pp_get_app_type2((const char *) slcf->app_type.data,
-            slcf->app_type.len);
+        context->app_type = pp_get_app_type2((const char *) slcf->autogenerated.app_type.data,
+            slcf->autogenerated.app_type.len);
         if (context->app_type == PAT_NONE) {
             return NGX_DECLINED;
         }
